/**
 * @file
 * Provides Mason loader.
 */

(function ($, Drupal, drupalSettings, _d, _doc) {

  'use strict';

  var _id = 'mason';
  var _mounted = _id + '--on';
  var _element = '.' + _id + ':not(.' + _mounted + ')';

  /**
   * Mason utility functions.
   *
   * @param {HTMLElement} elm
   *   The HTML element.
   */
  function doMason(elm) {
    var $elm = $(elm);
    var options = $.extend({}, drupalSettings.mason, $elm.data('mason'));

    $elm.mason(options);
  }

  /**
   * Attaches jumper behavior to HTML element identified by .mason.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.mason = {
    attach: function (context) {

      context = context || _doc;

      // Context is unreliable with AJAX contents.
      context = context instanceof HTMLDocument ? context : _doc;

      var elms = context.querySelectorAll(_element);
      if (elms.length > 0) {
        _d.once(_d.forEach(elms, doMason, context));
      }

    }
  };

})(jQuery, Drupal, drupalSettings, dBlazy, this.document);
