<?php

/**
 * @file
 * Hooks and preprocess functions for the Mason module.
 */

use Drupal\blazy\Blazy;
use Drupal\mason\MasonDefault;

/**
 * Prepares variables for mason.html.twig templates.
 */
function template_preprocess_mason(&$variables) {
  $element = $variables['element'];
  foreach (MasonDefault::themeProperties() as $key) {
    $variables[$key] = isset($element["#$key"]) ? $element["#$key"] : [];
  }

  // Prepare attributes, and add the configuration as JSON object.
  $settings = &$variables['settings'];
  $settings += MasonDefault::htmlSettings();
  $id = Blazy::getHtmlId('mason', $settings['id']);

  $attributes = &$variables['attributes'];
  $attributes['id'] = $id;

  $attributes['data-mason'] = $variables['json'];
  Blazy::containerAttributes($attributes, $settings);

  // Pass settings and attributes to twig.
  $classes = empty($variables['attributes']['class']) ? [] : $variables['attributes']['class'];
  if (strpos($id, 'mason-') !== FALSE) {
    $classes[] = str_replace('mason-', 'mason--', $id);
  }

  $variables['attributes']['class'] = array_merge(['mason'], $classes);
}

/**
 * Prepares variables for mason-box.html.twig templates.
 */
function template_preprocess_mason_box(&$variables) {
  foreach (['delta', 'item', 'settings'] as $key) {
    $default = $key == 'delta' ? 0 : [];
    $variables[$key] = isset($variables['element']["#$key"]) ? $variables['element']["#$key"] : $default;
  }

  // Boxes may have captions.
  $item = &$variables['item'];
  foreach (['alt', 'category', 'data', 'link', 'overlay', 'title'] as $key) {
    $item['caption'][$key] = isset($item['caption'][$key]) ? $item['caption'][$key] : [];
  }

  $settings        = $variables['settings'];
  $item['box']     = isset($item['box']) ? $item['box'] : [];
  $item['caption'] = array_filter($item['caption']);

  $variables['fillers'] = !empty($settings['fillers']) && ($variables['delta'] > $settings['fillers']);
  $classes = empty($variables['attributes']['class']) ? [] : $variables['attributes']['class'];
  $variables['attributes']['class'] = array_merge([($variables['fillers'] ? 'mason__fill' : 'mason__box'), 'box'], $classes);
}
